//----------------------------------------------
// Yanghai Tsin 
// November 1st, 1997
//----------------------------------------------
#include "stdafx.h"
#include "cmil.h"


CMil::CMil()
{
	m_bIsMilInitialized=FALSE;
	MilImageDisplay=0;
	MilImageGrab=0;
	MilApplication=NULL;
	MilDigitizer=NULL;
	MilDisplay=NULL;
	MilSystem=NULL;

	m_bColor=TRUE;
	m_bDisplay=TRUE;
	m_dScale=1.0;
	m_bTimerReset=FALSE;
	ChildColorRed=0;
	ChildColorGreen=0;
	ChildColorBlue=0;
		

}

CMil::~CMil()
{

}


bool CMil::Initialize(bool bColor, double dScale, bool bDisplay)
{

	m_bColor=bColor;
	m_dScale=dScale;
	m_bDisplay=bDisplay;

	// Allocate an application
	MappAlloc(M_DEFAULT,&MilApplication);

	//MappControl(M_ERROR,M_PRINT_DISABLE);


	// Allocate a system	
 	MsysAlloc(M_SYSTEM_SETUP,M_DEF_SYSTEM_NUM,M_COMPLETE+M_NO_DDRAW,&MilSystem);

	//Allocate display
	MdispAlloc(MilSystem,M_DEFAULT, M_DISPLAY_SETUP, M_DEFAULT,&MilDisplay);
	// Inquire number of digitizers available on the system

    // Disable scroll bars on display
    MdispControl(MilDisplay,M_WINDOW_SCROLLBAR,M_DISABLE);
	MdispControl(MilDisplay,M_WINDOW_ZOOM,M_ENABLE);
	MdispControl(MilDisplay,M_WINDOW_RESIZE	,M_ENABLE);
	MdispControl(MilDisplay,M_WINDOW_MENU_BAR_CHANGE,M_ENABLE);
	MdispControl(MilDisplay,M_WINDOW_MENU_BAR,M_DISABLE);
	MdispControl(MilDisplay,M_WINDOW_OVERLAP,M_ENABLE);
	MdispControl(MilDisplay,M_WINDOW_TITLE_BAR_CHANGE,M_ENABLE);
	MdispControl(MilDisplay,M_WINDOW_TITLE_BAR,M_DISABLE);
	MdispZoom(MilDisplay, 1, 1);

	MsysInquire(MilSystem,M_SYS_DIGITIZER_NUM,&NumberOfDigitizer);

	
   // Digitizer is available
   if (NumberOfDigitizer)	
	{
      // Allocate a digitizer
		MdigAlloc(MilSystem,M_DEFAULT,M_CAMERA_SETUP,M_DEFAULT,&MilDigitizer);
		// Stop live grab when window is disable
    	MsysControl(MilSystem,M_STOP_LIVE_GRAB_WHEN_DISABLED,M_ENABLE);
		// Inquire digitizer informations
  		MdigInquire(MilDigitizer,M_SIZE_X,&DigSizeX);
		MdigInquire(MilDigitizer,M_SIZE_Y,&DigSizeY);  	
		MdigInquire(MilDigitizer,M_SIZE_BAND,&Band);
		
		if (DigSizeX > M_DEF_IMAGE_SIZE_X_MAX) 
		   DigSizeX = M_DEF_IMAGE_SIZE_X_MAX;
		if (DigSizeY > M_DEF_IMAGE_SIZE_Y_MAX) 
			DigSizeY = M_DEF_IMAGE_SIZE_Y_MAX;
	}
   // Digitizer is not available
   else					
	{
		SizeX = M_DEF_IMAGE_SIZE_X_MIN; 
		SizeY = M_DEF_IMAGE_SIZE_Y_MIN;
		Band  =	M_DEF_IMAGE_NUMBANDS_MIN;
	}

	if(NumberOfDigitizer)
	{
			SizeX=DigSizeX;
			SizeY=DigSizeY;
	}


	m_bIsMilInitialized=TRUE;


	ResetMil();

	MgraControl(M_DEFAULT, M_BACKGROUND_MODE, M_TRANSPARENT);
	

	
	//GrabIsStarted = ;
	
	
	return TRUE;

}

void CMil::Close(void)
{

	if(MilImageDisplay)
	{
		MdispDeselect(MilDisplay,MilImageDisplay);
		if(ChildColorRed)	
		{
			MbufFree(ChildColorRed);
			ChildColorRed=0;
		}
		if(ChildColorGreen)
		{	
			MbufFree(ChildColorGreen);
			ChildColorGreen=0;
		}
		if(ChildColorBlue)	
		{
			MbufFree(ChildColorBlue);
			ChildColorBlue=0;
		}

		MbufFree(MilImageDisplay);
		MilImageDisplay=0;
	}
	
	if(MilImageGrab) MbufFree(MilImageGrab);
	MilImageGrab=0;
	// Free the display
	if (MilDisplay) MdispFree(MilDisplay);
	
	MilDisplay = M_NULL;

 	// MIL: Write your code that will be executed on application exit
	/////////////////////////////////////////////////////////////////////////	

	if (MilDigitizer)    MdigFree (MilDigitizer);	// Free the digitizer
	if (MilSystem) 	   MsysFree (MilSystem);		// Free the system
	if (MilApplication)  MappFree (MilApplication);	// Free the application
                        
	/////////////////////////////////////////////////////////////////////////
	// MIL: Write your code that will be executed on application exit
	m_bIsMilInitialized=FALSE;
	MilImageDisplay=0;
	MilImageGrab=0;
	MilApplication=NULL;
	MilDigitizer=NULL;
	MilDisplay=NULL;
	MilSystem=NULL;

	m_bColor=TRUE;
	m_bDisplay=TRUE;
	m_dScale=0.5;
	m_bTimerReset=FALSE;

}

int CMil::GetX()
{
	if(m_bIsMilInitialized)
	{
		return (int) (SizeX*m_dScale);
	}
	else
		return -1;

}

int CMil::GetY()
{
 	if(m_bIsMilInitialized)
	{
		return (int) (SizeY*m_dScale);
	}
	else
		return -1;


}

bool CMil::GetBuffer(LPVOID buffer, MIL_ID * MilImage)
{
  	if(MilImage==NULL)
		return FALSE;
	if(m_bIsMilInitialized)
	{
		if(m_bColor)
		{
			MbufGetColor(*MilImage,M_RGB24+M_PACKED,M_ALL_BAND,buffer);
		}
		else
		{
			MbufGet(*MilImage,buffer);
		}
		return TRUE;
	}

	return FALSE;

}


bool CMil::ResetMil(void)
{
	if(m_bIsMilInitialized)
	{
		/* Wait last grab end. */
		MdigGrabWait(MilDigitizer, M_GRAB_END);
		if(m_bTimerReset)
		{
			MappTimer(M_TIMER_READ, &Time);
			
			m_bTimerReset=FALSE;
		}

		if(ChildColorRed)	
		{
			MbufFree(ChildColorRed);
			ChildColorRed=0;
		}
		if(ChildColorGreen)
		{	
			MbufFree(ChildColorGreen);
			ChildColorGreen=0;
		}
		if(ChildColorBlue)	
		{
			MbufFree(ChildColorBlue);
			ChildColorBlue=0;
		}
		
				
		if(MilImageDisplay)
		{
			MdispDeselect(MilDisplay, MilImageDisplay);;
			MbufFree(MilImageDisplay);
			MilImageDisplay=0;
		}

		
		MdigHalt(MilDigitizer);
		if(MilImageGrab) MbufFree(MilImageGrab);
		MilImageGrab=0;

		if(NumberOfDigitizer)
			BufferAttributes=M_IMAGE+M_GRAB;
		else
			BufferAttributes=M_IMAGE;
			

		if(m_bColor)
		{
			MbufAllocColor(MilSystem,Band,(long)(SizeX*m_dScale),(long)(SizeY*m_dScale),M_DEF_IMAGE_TYPE,	
			BufferAttributes,
			&MilImageGrab);
		}
		else
		{
			MbufAlloc2d(MilSystem,(long)(SizeX*m_dScale),(long)(SizeY*m_dScale),8+M_UNSIGNED,	
			BufferAttributes,
  			&MilImageGrab);
		}
		if(!MilImageGrab)
			return FALSE;
		MbufClear(MilImageGrab,0);


		BufferAttributes=M_IMAGE+M_PROC+M_DISP+M_GRAB;
		
		if(m_bColor)
		{
  			MbufAllocColor(MilSystem,Band,(long)(SizeX*m_dScale),(long)(SizeY*m_dScale),M_DEF_IMAGE_TYPE,	
				BufferAttributes,
  				&MilImageDisplay);
		}
		else
		{
			MbufAlloc2d(MilSystem,(long)(SizeX*m_dScale),(long)(SizeY*m_dScale),8+M_UNSIGNED,//M_DEF_IMAGE_TYPE,	
				BufferAttributes,
  				&MilImageDisplay);
		}


		if(!MilImageDisplay)
			return FALSE;

		// Clear the buffer 
		ClearDisplayBuffer();
		
		//MbufClear(MilImageCont,0xff);
		if(m_bColor){
			MbufChildColor(MilImageDisplay, M_RED, &ChildColorRed);
			MbufChildColor(MilImageDisplay, M_GREEN, &ChildColorGreen);
			MbufChildColor(MilImageDisplay, M_BLUE, &ChildColorBlue);
		}
		
		MdispSelect(MilDisplay, MilImageDisplay);

		MdigControl(MilDigitizer, M_GRAB_SCALE,m_dScale);
		MdigControl(MilDigitizer, M_GRAB_MODE, M_ASYNCHRONOUS);
		MdigControl(MilDigitizer,M_GRAB_FAIL_CHECK,M_DISABLE);

		MdigGrab(MilDigitizer, MilImageGrab);
		MdigGrab(MilDigitizer, MilImageGrab);
		MappTimer(M_TIMER_RESET, M_NULL);
		m_bTimerReset=TRUE;

		return TRUE;



	}

	return FALSE;
} 

double CMil::GetScale()
{
	return m_dScale;

}

bool CMil::GetColor()
{
	return m_bColor;

}

bool CMil::GetDisplay()
{
	return m_bDisplay;
}


bool CMil::SetScale(double dScale)
{
	if(m_bIsMilInitialized)
	{
		m_dScale=dScale;
		//return ResetMil(m_pView);
		return TRUE;
		
	}
	else
		return FALSE;

}

bool CMil::SetColor(bool bColor)
{ 	
	if(m_bIsMilInitialized)
	{
		m_bColor=bColor;
		return TRUE;
		//return ResetMil(m_pView);
	}
	else
		return FALSE;


}


bool CMil::SetDisplay(bool bDisplay)
{
	if(m_bIsMilInitialized)
	{
		m_bDisplay=bDisplay;
		return TRUE;
	}
	else
		return FALSE;

} 

bool CMil::GetLatestImage(char * buffer)
{
	if(m_bIsMilInitialized)
	{
		MdigGrab(MilDigitizer, MilImageGrab);
		GetBuffer( (LPVOID)buffer, &MilImageGrab);
		return TRUE;

	}
	
	return
		FALSE;

}

bool CMil::PutBuffer(void * buffer)
{

	if(m_bIsMilInitialized)
	{
		if(!m_bDisplay) 
			return TRUE;
		if(m_bColor)
		{
			MbufPutColor(MilImageDisplay,M_RGB24+M_PACKED,M_ALL_BAND,buffer);
		}
		else
		{
			MbufPut(MilImageDisplay,buffer);
		}
		return TRUE;
	}

	return FALSE;

}

/*bool CMil::NewMilDisplay(MIL_ID * MilDisplay)
{
	MdispAlloc(MilSystem,M_DEFAULT, M_DISPLAY_SETUP, M_DEFAULT,&MilDisplay);
}
  */

void CMil::SetForegroundColor(DWORD color)
{
	m_dColorValue=color;
	if(!m_bColor)
		MgraColor(M_DEFAULT, color);

}

bool CMil::TextOut(long x, long y, char * string)
{
	if(m_bIsMilInitialized)
	{

		if(m_bColor)
		{
			MgraColor(M_DEFAULT, LOBYTE(HIWORD(m_dColorValue)));
			MgraText(M_DEFAULT, ChildColorRed, x, y, string);
			MgraColor(M_DEFAULT, HIBYTE(LOWORD(m_dColorValue)));
			MgraText(M_DEFAULT, ChildColorGreen, x, y, string);
			MgraColor(M_DEFAULT, LOBYTE(LOWORD(m_dColorValue)));
			MgraText(M_DEFAULT, ChildColorBlue, x, y, string);
		}
		else
			MgraText(M_DEFAULT, MilImageDisplay, x, y, string);
		return TRUE;
	}
	else
		return FALSE;

}

bool CMil::FrameRect(long XStart, long YStart, long XEnd, long YEnd)
{
	if(m_bIsMilInitialized)
	{
		if(m_bColor)
		{
			MgraColor(M_DEFAULT, LOBYTE(HIWORD(m_dColorValue)));
			MgraRect(M_DEFAULT, ChildColorRed,XStart, YStart, XEnd, YEnd);
			MgraColor(M_DEFAULT, HIBYTE(LOWORD(m_dColorValue)));
			MgraRect(M_DEFAULT, ChildColorGreen,XStart, YStart, XEnd, YEnd);
			MgraColor(M_DEFAULT, LOBYTE(LOWORD(m_dColorValue)));
			MgraRect(M_DEFAULT, ChildColorBlue,XStart, YStart, XEnd, YEnd);
		}
		else
			MgraRect(M_DEFAULT, MilImageDisplay,XStart, YStart, XEnd, YEnd);
		return TRUE;
	}
	else
		return FALSE;

}

bool CMil::ClearDisplayBuffer()
{
	if(m_bIsMilInitialized)
	{
		MbufClear(MilImageDisplay,0xff);
		return TRUE;
	}
	else
		return FALSE;

	 
}

