//#include <string.h>
#include "stdafx.h"
#include "CSerial.h"

// Declare in case of NOCOMM...
WINBASEAPI BOOL WINAPI SetCommState(HANDLE hFile, LPDCB lpDCB);


// Initialize internal data structures.

CSerial::CSerial(void)
{
	portHdl = INVALID_HANDLE_VALUE;
}


// Close the serial port on destruction.

CSerial::~CSerial(void)
{
	if (portHdl != INVALID_HANDLE_VALUE) CloseHandle(portHdl);	
}


// Open the port.
// The port is specified as "COM1".
// The mode string is something like "baud=9600 parity=N data=8 stop=1".

BOOL CSerial::Connect(LPCSTR portName, LPCSTR mode)
{
	char fullName[200];
	DCB portDCB;
	HANDLE temp;

	temp = portHdl;

	wsprintf(fullName, "\\\\.\\%s", portName);
	portHdl = CreateFile(fullName, GENERIC_READ | GENERIC_WRITE, 
						 0, NULL, OPEN_EXISTING, 0, NULL);
	/*portHdl = CreateNamedPipe(fullName,PIPE_ACCESS_DUPLEX,PIPE_TYPE_BYTE,
		1,4096,4096,NMPWAIT_USE_DEFAULT_WAIT,NULL);*/
	if (portHdl == INVALID_HANDLE_VALUE) {
		wsprintf(errStr, "Could not open port %s\nErr = %d", fullName,
				 GetLastError());
		AfxMessageBox(errStr);
		portHdl = temp;
		return(FALSE);
	}

	
	if(!GetCommState(portHdl, &portDCB))
	{
		wsprintf(errStr, "Could not retrieve port status %s\nErr = %d", fullName,
				 GetLastError());
		AfxMessageBox(errStr);
		return(FALSE);
	}

	
	if (!BuildCommDCB(mode, &portDCB)) {
		wsprintf(errStr, "Could not build DCB from mode string\nErr = %d",
				 GetLastError());
		AfxMessageBox(errStr);
		CloseHandle(portHdl);
		portHdl = INVALID_HANDLE_VALUE;
		return(FALSE);
	}

	
	if (!SetCommState(portHdl, &portDCB)) {
		wsprintf(errStr, "Could not set communications state\nErr = %d",
				 GetLastError());
		AfxMessageBox(errStr);
		CloseHandle(portHdl);
		portHdl = INVALID_HANDLE_VALUE;
		return(FALSE);
	}
	time_out_control.ReadIntervalTimeout = MAXDWORD; 
    time_out_control.ReadTotalTimeoutMultiplier = 0; 
	time_out_control.ReadTotalTimeoutConstant = 0; 
    time_out_control.WriteTotalTimeoutMultiplier = 0; 
    time_out_control.WriteTotalTimeoutConstant = 0;

	SetCommTimeouts(portHdl, &time_out_control);

	return(TRUE);
}


// Return the number of characters available to be read.

//BOOL CSerial::Length(DWORD* len)
//{

	
//	return PeekNamedPipe(portHdl, NULL, 0, NULL, len, NULL);
	
	/*if(	portHdl != INVALID_HANDLE_VALUE)
	{
		return ReadFile(portHdl, errStr, 0, len, NULL);
	}
	*/

	
//}


// Read a single character. Blocks on read, unless timeouts have been set.

/*
DWORD CSerial::GetChar(char* c)
{
	//char c;
	DWORD num_read;

	ReadFile(portHdl, c, 1, &num_read, NULL);
	return num_read;
	
}
*/
/*DWORD CSerial::GetStr(char* c, DWORD len)
{
	DWORD num_read;

	ReadFile(portHdl, c, len, &num_read, NULL);
	return num_read;

}
*/
// Write a single character.

/*void CSerial::PutChar(char c)
{
	DWORD num_written;

	WriteFile(portHdl, &c, 1, &num_written, NULL);
}
*/

// Write a null-terminated string.

BOOL CSerial::SerialStringOut(char *str, DWORD* num_written)
{
						
	if(num_written==NULL)
		num_written=&dwDefaultDword;
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		return WriteFile(portHdl, str, strlen(str), num_written, NULL);
	}

	return FALSE;


}


// Force pending characters to be output.

BOOL CSerial::FlushOutputBuffer(void)
{
	//return FlushFileBuffers(portHdl);

	return PurgeComm(portHdl,PURGE_TXCLEAR);
}

BOOL CSerial::FlushInputBuffer(void)
{
	//char	c[255];
	/*DWORD	num_read;
	DWORD len;

	if(Length(&len))
	{
		return ReadFile(portHdl, errStr, len, &num_read, NULL);
	}

	return FALSE;*/
	//return PurgeComm(portHdl, PURGE_RXCLEAR);
	DWORD len,num_read;

	Length(&len);
	return ReadFile(portHdl, errStr, len, &num_read, NULL);
}

BOOL CSerial::Disconnect()
{
	if (portHdl != INVALID_HANDLE_VALUE) 
	{
		if(FlushOutputBuffer())
			if(FlushInputBuffer())
				return CloseHandle(portHdl);
	}
	return FALSE;
}

BOOL CSerial::SerialBytesOut(CHAR* buffer, DWORD charCnt, DWORD* charOutCnt)
{
	if(charOutCnt==NULL)
		charOutCnt=&dwDefaultDword;
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		return WriteFile(portHdl, buffer, charCnt, charOutCnt, NULL);
	}
	return FALSE;
}

BOOL CSerial::SerialBytesIn(CHAR * buffer, DWORD charCnt, DWORD * charRead, DWORD timeout)
{
	DWORD len, start_time;

	len=0;
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		//immediately return with read bytes
		if(timeout==0)
		{
			return 	ReadFile(portHdl, buffer, charCnt, charRead, NULL);
		}

		//wait until really get all desired bytes

		if(timeout==MAXDWORD)
		{
		/*	do
			{
				if(!ReadFile(portHdl, buffer, charCnt-len, charRead, NULL))
					return FALSE;												   
				buffer+=*charRead;
				len+=*charRead;

			} while(len<charCnt);

			*charRead=len;*/
			do
			{
				if(!Length(&len))
				return FALSE;
			}while(len<charCnt);
			if(!ReadFile(portHdl, buffer, charCnt, charRead, NULL))
				return FALSE;

			return TRUE;
		}

		start_time=::GetTickCount();

		/*do
		{
			if(!ReadFile(portHdl, buffer, charCnt-len, charRead, NULL))
				return FALSE;
			buffer+=*charRead;
			len+=*charRead;
		} while(len<charCnt && (::GetTickCount()-start_time)<timeout);

		*charRead=len;
		if(len>=charCnt)*/
		do
		{
			if(!Length(&len))
			return FALSE;
		}while(len<charCnt && (::GetTickCount()-start_time)<timeout);
		if(len>=charCnt)
		{
			if(!ReadFile(portHdl, buffer, charCnt, charRead, NULL))
				return FALSE;
			return 	TRUE;
		}
	}

	return FALSE;

}

BOOL CSerial::ReadSerialLine(CHAR * buffer, DWORD timeout, DWORD* charsRead)
{
	*charsRead=0;
	
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		while(TRUE)
		{
			if(!SerialBytesIn(buffer,1, &dwDefaultDword,timeout) || dwDefaultDword<1)
				return FALSE;
			if(*buffer=='\n')
			{
				*buffer=='\0';
				return TRUE;
			}
			else
			{
				buffer++;
				(*charsRead)++;
			}
		}
	}

	return FALSE;
}

BOOL CSerial::GetSignedShort(signed short * SHORTval, DWORD timeout)
{
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		if(!SerialBytesIn( (CHAR*) (SHORTval)+1, 1, &dwDefaultDword, timeout))
			return FALSE;
		if(!SerialBytesIn( (CHAR*) (SHORTval), 1, &dwDefaultDword,timeout))
			return FALSE;
		return TRUE;
	}

	return FALSE;
}

BOOL CSerial::PutSignedShort(signed short * SHORTval)
{
	if(	portHdl != INVALID_HANDLE_VALUE)
	{	
		if(!SerialBytesOut( (((CHAR *) SHORTval)+1), 1))
			return FALSE;
		if(!SerialBytesOut( ( (CHAR *) SHORTval), 1))
			return FALSE; 
		return TRUE;
	}

	return FALSE;

}

BOOL CSerial::PutUnsignedShort(unsigned short * SHORTVal)
{
	if(	portHdl != INVALID_HANDLE_VALUE)
	{	
		if(!SerialBytesOut( (((char *) SHORTVal)+1), 1))
			return FALSE;
		if(!SerialBytesOut( ( (char *) SHORTVal), 1))
			return FALSE; 
		return TRUE;
	}

	return FALSE;

}

BOOL CSerial::GetUnsignedShort(unsigned short * USHORTVal, DWORD timeout)
{
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		if(!SerialBytesIn( (((CHAR *) USHORTVal)+1), 1, &dwDefaultDword, timeout))
			return FALSE;
		if(!SerialBytesIn( ( (CHAR *) USHORTVal), 1, &dwDefaultDword,timeout))
			return FALSE;
		return TRUE;
	}

	return FALSE;

}

BOOL CSerial::GetSignedLong(signed long * LONGVal, DWORD timeout)
{
	long i;

	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		//LONGVal = (signed long *) (((unsigned char *) LONGVal) + 3);
				
		for (i=3; i>=0; i--)
		{ 
			if(!SerialBytesIn( ((char *) LONGVal)+i, 1, &dwDefaultDword,timeout))
				return FALSE;
			//((unsigned char *)LONGVal)--;
		}
		return TRUE;
	}
	
	return FALSE;

}

BOOL CSerial::PutSignedLong(signed long * LONGVal)
{
	int i;
	//((CHAR*) LONGVal) +=3;
	if(	portHdl != INVALID_HANDLE_VALUE)
	{
		for(i=3;i>=0;i--)
		{
			 if(!SerialBytesOut( ((char *) LONGVal+i), 1))
				return FALSE;
			 //LONGVal--;
		}
		return TRUE;
	}
	return FALSE;

}

BOOL CSerial::Length(DWORD * len)
{
	COMSTAT cs;	 
	DWORD temp;

	if( portHdl != INVALID_HANDLE_VALUE)
	{
		if(len==NULL)
			len=&dwDefaultDword;
		if(! ClearCommError(portHdl, &temp, &cs))
			return FALSE;
		else
		{
			*len=cs.cbInQue;
			return TRUE;
		}
	}
	
	return FALSE;


}
