// CVisca98 - new visca control code
// By Alan Lipton (based on code by Mike Blackwell and Bob Collins)
//
// Copyright July 1998 - Carnegie Mellon University. All rights reserved
//

#include "stdafx.h"
#include "CVisca98.h"
#include "winprintf.h"
//#include <winbase.h>




//--------------------------------------------------------------------------


CVisca98::CVisca98(void)
{
	port = NULL;
	PortName = "";
	NUM_PACKETS_TO_DUMP = 10;
	DEBUG_PACKETS = 0;
	ALLOW_ERROR_MESSAGES = 0;
	strcpy(ERROR_FILENAME, DATA_DIR);
	strcat(ERROR_FILENAME,"ViscaERR.log");
	RING_BUFFER_LEN = RBLEN ;
	ViscaOpen = false;
	CameraType = EVI_370;
	Zoom = -1;
	ZoomSpeed = 7;
	Focus = 0;
	FocusSpeed = 7;
	Shutter = 0;
	Iris = 0;
	Gain = 0;
	KeyLock = false;
	TimeDisplay = false;
	DateDisplay = false;
}


CVisca98::~CVisca98(void)
{
	FreePort();
}


void CVisca98::FreePort(void)
{
	if (port) {
		delete port;
		port = NULL;
	}
	PortName = "";
}


BOOL CVisca98::InitializePort(LPCSTR portName)
{
	if (port) FreePort();

	PortName = "";
	port = new CSerial;
	if (port) {
		if (!port->Connect(portName, VISCA_PORT_MODE)) {
			delete port;
			port = NULL;
			return(FALSE);
		}
		PortName = portName;
		return(TRUE);
	}
	return(FALSE);
}

bool CVisca98::Close()
{
	port->Disconnect();
	return true;
}


// Set up the port, then initialize communications

bool CVisca98::Initialize(LPCSTR portName)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int repl, len;

// Opening serial line
	wgPrintf("Trying to open Visca98 on %s", portName);
	init_packet_ring_buffer();
	if (!InitializePort(portName)) {
		wgPrintf("Cannot open %s", portName);
		return(FALSE);	
	}

// Sending network initialisation command
	buffer[0] = 0x30;
	buffer[1] = 0x01;
	SendMessage(-1, buffer, 2, -1);		
	// Note no format byte
	// We don't get an ACK here, but another set address broadcast

	if (!RecvMessage(&repl, buffer, &len)) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Timeout initializing Visca network");
		dump_packet_ring_buffer_to_file("Timeout initializing Visca network");
		FreePort();
		return(FALSE);
	}

// Send IF_CLEAR command
	buffer[0] = 0x00;
	buffer[1] = 0x01;
	SendMessage(-1, buffer, 2, 1);		
	// format 1
	// Wait to read reply (which is an echo of the IF_clear command)

	if (!RecvMessage(&repl, buffer, &len)) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Timeout doing IF_Clear on Visca network");
		dump_packet_ring_buffer_to_file("Timeout doing IF_Clear on Visca network");
		FreePort();
		return(FALSE);
	}
	

	return(TRUE);
}

bool CVisca98::Open(LPCSTR portName)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

// CAN WE OPEN THE VISCA NETWORK?
	if (!ViscaOpen) {
		ViscaOpen = Initialize(portName);
	}
	if (!ViscaOpen) {
		wgPrintf("Error connecting to Visca Network");
		return false;
	}
	wgPrintf("Connection to Visca on %s established", portName);

// IDENTIFY CAMERA
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Establishing camera type");

	CameraType = IdentifyCamera(1);
	if (ALLOW_ERROR_MESSAGES) {
		switch(CameraType) {
		case EVI_330:
			wgPrintf("Camera type is EVI-330");
			break;
		case EVI_370:
			wgPrintf("Camera type is EVI-370");
			break;
		case UNKNOWN_CAMERA:
		default:
			wgPrintf("Camera type is UNKNOWN");
			break;
		}
	}

//	SetKeyLock(1, false);

// MAKE SURE POWER IS UP
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Powering up");
	buffer[0] = 0x04;
	buffer[1] = 0x00;
	buffer[2] = 0x02;
	SendMessage(1, buffer, 3);		
	if (!WaitAck(1, "Power on")) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Error on power up");
		return false;
	}
	if (!WaitComp(1, "Power on", buffer, &len)) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Error on power up");
		return false;
	}

// PUT IN MANUAL FOCUS MODE FOR SAFETY	
	SetFocusMode(1, FOCUS_MODE_MANUAL);

	SetTimeDisplay(1, DISPLAY_OFF);
//	if (CameraType == EVI_370) {
//		buffer[0] = 0x04;
//		buffer[1] = 0x38;
//		buffer[2] = 0x03;
//		SendMessage(1, buffer, 3);		
//		if (!WaitAck(1, "Manual focus")) {
//			if (ALLOW_ERROR_MESSAGES) wgPrintf("Error putting camera in manual focus mode");
//			return false;
//		}
//		if (!WaitComp(1, "Manual Focus", buffer, &len)) {
//			if (ALLOW_ERROR_MESSAGES) wgPrintf("Error putting camera in manual focus mode");
//			return false;
//		}
//	}

// GET CURRENT CAMERA SETTINGS	
	GetCameraState();


	return	true;
}

bool	CVisca98::GetCameraState()
{
	if (ALLOW_ERROR_MESSAGES) wgPrintf("-------------CAMERA STATE-------------");

	Zoom = GetZoom(1);

	FocusMode = GetFocusMode(1);

	if (CameraType == EVI_370) {
		Focus = GetFocus(1);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Initial Focus = %d", Focus);
	}
	else {
		Focus = 0;
	}
	WBMode = GetWBMode(1);

	AEMode = GetAEMode(1);
		
	if (CameraType == EVI_370) {
		ECMode = GetECMode(1);
	}

	Shutter = GetShutter(1);

	Iris = GetIris(1);

	Gain = GetGain(1);

	KeyLock = GetKeyLock(1);


	if (ALLOW_ERROR_MESSAGES) wgPrintf("--------------------------------------");

	return true;
}

int CVisca98::IdentifyCamera(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int repl, len;
	CString mess;

	buffer[0] = 0x00;
	buffer[1] = 0x02;
	SendMessage(camera, buffer, 2, 0x09);
	if (!RecvMessage(&repl, buffer, &len)) {
		dump_packet_ring_buffer_to_file("Timeout waiting for device inquiry");
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Timeout waiting for device inquiry");
		return UNKNOWN_CAMERA;
	}
	if (repl != camera) {
		CString err;
		err.Format("Device inquiry, expecting address %d, got %d", camera, repl);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Device inquiry, expecting address %d, got %d", camera, repl);
		return UNKNOWN_CAMERA;
	}
	if (len != 8) {
		CString err;
		err.Format("Device inquiry, expecting length 8, got %d", len);
		dump_packet_ring_buffer_to_file(err);	
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Device inquiry, expecting length 8, got %d", len);
		return UNKNOWN_CAMERA;
	}
	if (buffer[0] != 0x50) {
		CString err;
		err.Format("Device inquiry, expecting reply 0x50, got 0x%02x", buffer[0]);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Device inquiry, expecting reply 0x50, got 0x%02x", buffer[0]);
		return UNKNOWN_CAMERA;
	}
	mess.Format("Device %d inquiry:\nVendor = %02x%02x\nModel ID = %02x%02x\n"
				"ROM rev = %02x%02x\nMax socket = %02x", camera, buffer[1], buffer[2],
				buffer[3], buffer[4], buffer[5], buffer[6], buffer[7]);
	dump_packet_ring_buffer_to_file(mess);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Device %d inquiry: Vendor = %02x%02x, Model ID = %02x%02x, "
		"ROM rev = %02x%02x, Max socket = %02x", camera, buffer[1], buffer[2],
				buffer[3], buffer[4], buffer[5], buffer[6], buffer[7]);
	if (buffer[4] == 1) {
		return (CameraType = EVI_330);
	}
	if (buffer[4] == 4) {
		return (CameraType = EVI_370);
	}
	return UNKNOWN_CAMERA;
}


// Packetize up a message and send it. If dest is -1, broadcast to all stations.
// Message is sent in immediate mode (format 1), unless format is set.

BOOL CVisca98::SendMessage(int dest, const unsigned char *data, int length, int format)
{
	unsigned char byte, buffer[PACKET_MAX_LENGTH];
	char bufstr[PACKET_MAX_LENGTH*3+7];
	int i,j=0;
	DWORD	ret;
	char byte_out;

	if (!port) {
		dump_packet_ring_buffer_to_file("No port initialized for SendMessage!");
		if (ALLOW_ERROR_MESSAGES) AfxMessageBox("No port initialized for SendMessage!", MB_ICONEXCLAMATION);
		return(FALSE);
	}

	if (dest > 7) {
		CString err;
		err.Format("SendMessage destination out of range (%d)!", dest);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) AfxMessageBox(err, MB_ICONEXCLAMATION);
		return(FALSE);
	}

	if (dest < 0) {
		byte = 0x88;
	} else {
		byte = 0x80 | dest;
	}
	port->FlushInputBuffer();
	byte_out = (char)byte;
	buffer[j++] = byte_out;
	if (!port->SerialBytesOut(&byte_out, 1, &ret))
		wgPrintf("FUCKED UP A WRITE");		// Packet header

	if (format >= 0) {
		byte_out = (char)(format & 0xFF);
		buffer[j++] = byte_out;	
		if (!port->SerialBytesOut(&byte_out, 1, &ret))
			wgPrintf("BOLLOCKS");	// Format 1
	}

	for (i = 0; i < length; i++) {
		byte_out = (char)(data[i]);
		buffer[j++] = byte_out;
		//wgPrintf("Data %d %d", data[i], byte_out);
		if (!port->SerialBytesOut(&byte_out, 1, &ret))
			wgPrintf("NO WRITE");	// Format 1
	}

	byte_out = (char)0xFF;
	buffer[j++] = byte_out;
	if (!port->SerialBytesOut(&byte_out, 1, &ret))
		wgPrintf("STILL NO WRITE");				// Terminator

	sprintf(bufstr,"send: ");
	for (i=0; i < j; i++)
		sprintf(bufstr+6+(i*3)," %02x",buffer[i]);
    add_to_packet_ring_buffer(bufstr);
	if (DEBUG_PACKETS)
		wgPrintf("%s",bufstr);
	return(FALSE);
}


void CVisca98::SendInquiry(int dest, unsigned char inq)
{
	unsigned char buffer[PACKET_MAX_LENGTH];

	buffer[0] = 0x04;
	buffer[1] = inq;
	SendMessage(dest, buffer, 2, 0x09);
}


// Receive the next packet. Data buffer must be allocated.
// If timeout, returns false.

BOOL CVisca98::RecvMessage(int *src, unsigned char *data, int *length)
{
	unsigned char *ptr, buffer[PACKET_MAX_LENGTH];
	char bufstr[PACKET_MAX_LENGTH*3+7];
	int i, j=0;
	short c;
	char cc;
	DWORD ret;

	if (!port) {
		dump_packet_ring_buffer_to_file("No port initialized for RecvMessage!");
		if (ALLOW_ERROR_MESSAGES) AfxMessageBox("No port initialized for RecvMessage!", MB_ICONEXCLAMATION);
		return(FALSE);
	}


	ptr = data;
	*length = 0;

	// Wait for packet header
	for (;;) {
		if (!port->SerialBytesIn(&cc, 1, &ret, PACKET_TIMEOUT)) {
			wgPrintf("No reply from Camera");
			return false;
		}
//			wgPrintf("Couldn't read");
//		c = port->GetChar(PACKET_TIMEOUT);
		buffer[j++] = (unsigned char)cc;
		c = (short)(unsigned char)cc;
//		wgPrintf("c %x cc %x", c, cc);
		if (c < 0) return(FALSE);
		if (c == 0x88) {
			*src = -1;
			break;
		}
		if ((c & 0x8F) == 0x80) {
			*src = (c & 0x70) >> 4;
			break;
		}
	}

	// Get chars until the terminator. If we fill the buffer,
	// throw out chars.
	for (;;) {
		if (!port->SerialBytesIn(&cc, 1, &ret, PACKET_TIMEOUT)) {
			wgPrintf("Partial response from camera");
			return false;
		}
		buffer[j++] = (unsigned char)cc;
		c = (short)(unsigned char)cc;
		if (c < 0) return(FALSE);
		if (c == 0xFF) break;
		if (*length < PACKET_MAX_LENGTH) {
			*ptr++ = (c & 0xFF);
			(*length)++;
		}
	}

	
	sprintf(bufstr,"recv: ");
	for (i=0; i < j; i++)
		sprintf(bufstr+6+(i*3)," %02x",buffer[i]);
    add_to_packet_ring_buffer(bufstr);
	if (DEBUG_PACKETS)
		wgPrintf("%s",bufstr);

	return(TRUE);
}


void CVisca98::ShowMessage(int src, unsigned char *data, int length)
{
	CString mess, temp, temp2;
	int i;

	temp = "";
	for (i = 0; i < length; i++) {
		temp2.Format("%02x ", data[i]);
		temp += temp2;
	}

	mess.Format("Message from source %d:\n%s", src, temp);
	dump_packet_ring_buffer_to_file(mess);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Message from source %d:\n%s", src, temp);
}


// Wait for an ACK packet from the destination

BOOL CVisca98::WaitAck(int dest, LPCSTR command)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int repl, len;

	
	if (!RecvMessage(&repl, buffer, &len)) {
		CString err;
		err.Format("Timeout waiting for ACK from %s", command);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Timeout waiting for ACK from %s", command);
		return(FALSE);
	}

	if (repl != dest) {
		CString err;
		err.Format("Received ACK from wrong address for %s\n"
				   "Wanted %d, got %d", command, dest, repl);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Received ACK from wrong address for %s, Wanted %d, got %d", command, dest, repl);		
		ShowMessage(repl, buffer, len);
		return(FALSE);
	}

	if (len < 1) {
		CString err;
		err.Format("Reply ACK packet too short for %s", command);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Reply ACK packet too short for %s", command);
		ShowMessage(repl, buffer, len);
		return(FALSE);
	}

	if ((buffer[0] & 0xF0) != 0x40) {
		CString err;
		err.Format("Did not receive expected ACK for %s", command);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Did not receive expected ACK for %s", command);
		ShowMessage(repl, buffer, len);
		return(FALSE);
	}
    

	return(TRUE);
}


// Wait for a completion packet from the destination.
// If there's any data, stuff it in to the buffer.

BOOL CVisca98::WaitComp(int dest, LPCSTR command, unsigned char *data, int *length)
{
	unsigned char buffer[PACKET_MAX_LENGTH], *ptr;
	int repl, len, i;



	*length = 0;
	ptr = data;

	
	if (!RecvMessage(&repl, buffer, &len)) {
		CString err;
		err.Format("Timeout waiting for COMP from %s", command);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Timeout waiting for COMP from %s", command);
		return(FALSE);
	}

	
	if (repl != dest) {
		CString err;
		err.Format("Received COMP from wrong address for %s\n"
				   "Wanted %d, got %d", command, dest, repl);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Received COMP from wrong address for %s, Wanted %d, got %d", command, dest, repl);
		ShowMessage(repl, buffer, len);
		return(FALSE);
	}

	if (len < 1) {
		CString err;
		err.Format("Reply COMP packet too short for %s", command);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Reply COMP packet too short for %s", command);
		ShowMessage(repl, buffer, len);
		return(FALSE);
	}
    
    
	if ((buffer[0] & 0xF0) != 0x50) {
		CString err;
		err.Format("Did not receive expected COMP for %s", command);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Did not receive expected COMP for %s", command);
		ShowMessage(repl, buffer, len);
		return(FALSE);
	}
    

	*length = len - 1;
	for (i = 1; i < len; i++) {
		*ptr++ = buffer[i];
	}

	return(TRUE);
}


bool CVisca98::GetKeyLock(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	SendInquiry(camera, 0x17);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "KeyLock Inq")) return(KeyLock = false);
	if (!WaitComp(camera, "KeyLock Inq", buffer, &len)) return(KeyLock = false);

	if (len < 1) {
		CString err;
		err.Format("KeyLock Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("KeyLock Inquiry packet too short (%d)", len);
		return(KeyLock = false);
	}

	(KeyLock = (buffer[0] == 0x02));

	if (KeyLock) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("KeyLock: ON");
	}
	else {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("KeyLock: OFF");
	}
	return KeyLock;
}


// 0 = Auto, 1 = Manual

int CVisca98::GetFocusMode(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	SendInquiry(camera, 0x38);
	if (!WaitComp(camera, "AFMode Inq", buffer, &len)) return(-1);

	if (len < 1) {
		CString err;
		err.Format("AFMode Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AFMode Inquiry packet too short (%d)", len);
		return(FocusMode = -1);
	}

	if (buffer[0] == 0x02) {
		FocusMode = FOCUS_MODE_AUTO;
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus Mode: Auto");
		return(FocusMode);
	}
	if (buffer[0] == 0x03) {
		(FocusMode = FOCUS_MODE_MANUAL);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus Mode: Manual");
		return FocusMode;
	}

	CString err;
	err.Format("Unknown AF Mode Inquiry reply 0x%02x", buffer[0]);
	dump_packet_ring_buffer_to_file(err);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Unknown AF Mode Inquiry reply 0x%02x", buffer[0]);
	return(FocusMode = -1);
}


int CVisca98::GetWBMode(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	SendInquiry(camera, 0x35);
	if (!WaitComp(camera, "WBMode Inq", buffer, &len)) return(-1);

	if (len < 1) {
		CString err;
		err.Format("WBMode Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("WBMode Inquiry packet too short (%d)", len);
		return(WBMode = -1);
	}

	if (buffer[0] == 0x00) (WBMode = WBMODE_AUTO);
	if (buffer[0] == 0x01) (WBMode = WBMODE_INDOOR);
	if (buffer[0] == 0x02) (WBMode = WBMODE_OUTDOOR);
	if (buffer[0] == 0x03) (WBMode = WBMODE_ONE_PUSH);

	switch(WBMode) {
	case WBMODE_AUTO:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("White Balance Mode: Auto");
		return WBMode;
		break;
	case WBMODE_INDOOR:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("White Balance Mode: Indoor");
		return WBMode;
		break;
	case WBMODE_OUTDOOR:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("White Balance Mode: Outdoor");
		return WBMode;
		break;
	case WBMODE_ONE_PUSH:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("White Balance Mode: One Push");
		return WBMode;
		break;
	default:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("White Balance Mode: Unknown");
		break;
	}



	CString err;
	err.Format("Unknown WB Mode Inquiry reply 0x%02x", buffer[0]);
	dump_packet_ring_buffer_to_file(err);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Unknown AF Mode Inquiry reply 0x%02x", buffer[0]);
	return(WBMode = -1);
}


int CVisca98::GetZoom(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len, zoom; 

	SendInquiry(camera, 0x47);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "Zoom Inq")) return(-1);
	if (!WaitComp(camera, "Zoom Inq", buffer, &len)) return(-1);


	if (len < 4) {
		CString err;
		err.Format("Zoom Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Zoom Inquiry packet too short (%d)", len);
		return(-1);
	}

	zoom = ((buffer[1] & 0x0F) << 8) +
		   ((buffer[2] & 0x0F) << 4) +
		   (buffer[3] & 0x0F);
	Zoom = zoom;
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Zoom = %d", Zoom);
	return(Zoom);
}


int CVisca98::GetFocus(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len, focus; 

	if (CameraType == EVI_330) return (0);
	SendInquiry(camera, 0x48);
	if (!WaitComp(camera, "Focus Inq", buffer, &len)) return(-1);


	if (len < 4) {
		CString err;
		err.Format("Focus Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus Inquiry packet too short (%d)", len);
		return(-1);
	}

	focus = ((buffer[0] & 0x0F) << 12) +
		   ((buffer[1] & 0x0F) << 8) +
		   ((buffer[2] & 0x0F) << 4) + 
		   (buffer[3] & 0x0F);
	Focus = focus;
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus value = %d", Focus);
	return(Focus);
}


int CVisca98::GetAEMode(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	SendInquiry(camera, 0x39);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "AE Mode Inq")) return(AEMode = -1);
	if (!WaitComp(camera, "AE Mode Inq", buffer, &len)) return(AEMode = -1);

	if (len < 1) {
		CString err;
		err.Format("AE Mode Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AE Mode Inquiry packet too short (%d)", len);
		return(AEMode = -1);
	}

	if (buffer[0] == 0x00) (AEMode = AEMODE_FULL_AUTO);
	if (buffer[0] == 0x03) (AEMode = AEMODE_MANUAL);
	if (buffer[0] == 0x0A) (AEMode = AEMODE_SHUTTER);
	if (buffer[0] == 0x0B) (AEMode = AEMODE_IRIS);
	if (buffer[0] == 0x0D) (AEMode = AEMODE_BRIGHTNESS);
	switch(AEMode) {
	case AEMODE_FULL_AUTO:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AEMode: Auto");
		return AEMode;
		break;
	case AEMODE_MANUAL:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AEMode: Manual");
		return AEMode;
		break;
	case AEMODE_SHUTTER:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AEMode: Shutter");
		return AEMode;
		break;
	case AEMODE_BRIGHTNESS:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AEMode: Brightness");
		return AEMode;
		break;
	case AEMODE_IRIS:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AEMode: Iris");
		return AEMode;
		break;
	default:
		if (ALLOW_ERROR_MESSAGES) wgPrintf("AEMode: Unknown");
		break;
	}

	CString err;
	err.Format("Unknown AE Mode Inquiry reply 0x%02x", buffer[0]);
	dump_packet_ring_buffer_to_file(err);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Unknown AE Mode Inquiry reply 0x%02x", buffer[0]);
	return(AEMode = -1);
}

int CVisca98::GetECMode(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command not supported in EVI-330");
		return false;
	}

	SendInquiry(camera, 0x33);
	if (!WaitComp(camera, "EC Mode Inq", buffer, &len)) return(ECMode = -1);

	if (len < 1) {
		CString err;
		err.Format("EC Mode Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("EC Mode Inquiry packet too short (%d)", len);
		return(ECMode = -1);
	}

	if (buffer[0] == 0x02) (ECMode = ECMODE_ON);
	if (buffer[0] == 0x03) (ECMode = ECMODE_OFF);

	if (ECMode == ECMODE_ON) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("ECMode: ON");
		return ECMode;
	}
	if (ECMode == ECMODE_OFF) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("ECMode: OFF");
		return ECMode;
	}


	CString err;
	err.Format("Unknown EC Mode Inquiry reply 0x%02x", buffer[0]);
	dump_packet_ring_buffer_to_file(err);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Unknown EC Mode Inquiry reply 0x%02x", buffer[0]);
	return(ECMode = -1);
}


int CVisca98::GetShutter(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len, shut;

	SendInquiry(camera, 0x4A);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "Shutter Inq")) return(Shutter = 0);
	if (!WaitComp(camera, "Shutter Inq", buffer, &len)) return(Shutter = 0);

	if (len < 4) {
		CString err;
		err.Format("Shutter Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Shutter Inquiry packet too short (%d)", len);
		return(Iris = 0);
	}

	shut = ((buffer[2] & 0x0F) << 4) + (buffer[3] & 0x0F);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Shutter: %d", shut);

	return(Shutter = shut);
}

int CVisca98::GetIris(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len, iris;

	SendInquiry(camera, 0x4B);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "Iris Inq")) return(Iris = 0);
	if (!WaitComp(camera, "Iris Inq", buffer, &len)) return(Iris = 0);

	if (len < 4) {
		CString err;
		err.Format("Iris Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Iris Inquiry packet too short (%d)", len);
		return(Iris = 0);
	}

	iris = ((buffer[2] & 0x0F) << 4) + (buffer[3] & 0x0F);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Iris: %d", iris);

	return(Iris = iris);
}

	
int CVisca98::GetGain(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len, gain;

	SendInquiry(camera, 0x4C);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "Gain Inq")) return(Gain = 0);
	if (!WaitComp(camera, "Gain Inq", buffer, &len)) return(Gain = 0);

	if (len < 4) {
		CString err;
		err.Format("Gain Inquiry packet too short (%d)", len);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Gain Inquiry packet too short (%d)", len);
		return(Gain = 0);
	}

	gain = (buffer[3] & 0x0F);
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Gain: %d", gain);

	return(Gain = gain);
}

	
void CVisca98::SetKeyLock(int camera, BOOL on)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	buffer[0] = 0x04;
	buffer[1] = 0x17;
	buffer[2] = (on ? 0x02 : 0x00);
	SendMessage(camera, buffer, 3);
	if (CameraType == EVI_330)
		if (!WaitAck(camera, "KeyLock")) return;
	if (!WaitComp(camera, "KeyLock", buffer, &len)) return;
}


// Mode 0 = Auto, 1 = Indoor, 2 = Outdoor, 3 = One Push + Trigger

void CVisca98::SetWBMode(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if ((mode < 0) || (mode > 3)) {
		CString err;
		err.Format("White balance mode out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("White balance mode out of range (%d)", mode);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x35;
	buffer[2] = mode;
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "WhiteBalance")) return;
	if (!WaitComp(camera, "WhiteBalance", buffer, &len)) return;
	WBMode = mode;
	return;
//	if (mode != 3) return;
}

void CVisca98::SetWBTrigger(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;
	// One push mode, trigger it

	if (WBMode != WBMODE_ONE_PUSH) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in current WBMode");
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x10;
	buffer[2] = 0x05;
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "WhiteBalance Trigger")) return;
	if (!WaitComp(camera, "WhiteBalance Trigger", buffer, &len)) return;
}

void CVisca98::SetZoomIn(int camera, int speed)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	buffer[0] = 0x04;
	buffer[1] = 0x07;

//	if (speed == 0) speed = ZoomSpeed;

	if (CameraType == EVI_330) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Zooming in EVI-330");
		SetZoom(camera, 1023, true);
		return;
	}
	if (CameraType == EVI_370) {
		if ((speed < 2) || (speed > 7)) {
			buffer[2] = 0x02;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Zooming in (standard) EVI-370");
		}
		else {
			buffer[2] = 0x20 + speed;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Zooming in (speed = %d) EVI-370", speed);
		}
	}

	SendMessage(camera, buffer, 3);
//	if (CameraType == EVI_330) {
		if (!WaitAck(camera, "Zoom In")) return;
//	}
	if (!WaitComp(camera, "Zoom In", buffer, &len)) return;


}

void CVisca98::SetZoomOut(int camera, int speed)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

//	if (speed == 0) speed = ZoomSpeed;

	buffer[0] = 0x04;
	buffer[1] = 0x07;

	if (CameraType == EVI_330) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Zooming out EVI-330");
		SetZoom(camera, 0, true);
		return;
	}
	if (CameraType == EVI_370) {
		if ((speed < 2) || (speed > 7)) {
			buffer[2] = 0x03;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Zooming out (standard) EVI-370");
		}
		else {
			buffer[2] = 0x30 + speed;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Zooming out (speed = %d) EVI-370", speed);
		}
	}

	SendMessage(camera, buffer, 3);
//	if (CameraType == EVI_330) {
		if (!WaitAck(camera, "Zoom Out")) return;
//	}
	if (!WaitComp(camera, "Zoom Out", buffer, &len)) return;


}

void CVisca98::SetZoomStop(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		SetZoom(camera, GetZoom(1), true);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x07;
	buffer[2] = 0x00;
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Stop Zoom")) return;
	if (!WaitComp(camera, "Stop Zoom", buffer, &len)) return;
	return;

}

void CVisca98::SetZoom(int camera, int zoom, bool wait)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		wait = true;
	}

	if ((zoom < 0) || (zoom > 0x3FF)) {
		CString err;
		err.Format("Zoom out of range (%d)", zoom);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Zoom out of range (%d)", zoom);
		return;
	}
	if ((zoom > 0) && (zoom < 0x3ff)) {
		zoom++;
	}

	//REMOVE THE NEXT 6 LINES WHEN DONE TESTING
/*
	if (zoom > 500) {
		CString err;
		err.Format("Testing error reporting: zoom = %d", zoom);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) AfxMessageBox(err, MB_ICONEXCLAMATION);
	}
*/



	buffer[0] = 0x04;
	buffer[1] = 0x47;
	buffer[2] = 0x00;
	buffer[3] = (zoom & 0x0F00) >> 8;
	buffer[4] = (zoom & 0x00F0) >> 4;
	buffer[5] = (zoom & 0x000F);
	SendMessage(camera, buffer, 6);
	if (!WaitAck(camera, "Zoom")) return;
	if (wait) {
		if (!WaitComp(camera, "Zoom", buffer, &len)) return;
	}
}

/* Doesn't appear to work -- Bob
void CVisca98::StopZoom(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	buffer[0] = 0x04;
	buffer[1] = 0x07;
	buffer[2] = 0x00;
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "StopZoom")) return;
	if (!WaitComp(camera, "StopZoom", buffer, &len)) return;
}
*/

void CVisca98::SetAEMode(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if ((mode != AEMODE_FULL_AUTO) && (mode != AEMODE_SHUTTER) && (mode != AEMODE_BRIGHTNESS)) {
			CString err;
			err.Format("AE mode out of range (%d)", mode);
			dump_packet_ring_buffer_to_file(err);
			if (ALLOW_ERROR_MESSAGES) wgPrintf("AE mode out of range (%d)", mode);
			return;
		}
	}
	if (CameraType == EVI_370) {
		if ((mode < 0) || (mode > 4)) {
			CString err;
			err.Format("AE mode out of range (%d)", mode);
			dump_packet_ring_buffer_to_file(err);
			if (ALLOW_ERROR_MESSAGES) wgPrintf("AE mode out of range (%d)", mode);
			return;
		}
	}

	buffer[0] = 0x04;
	buffer[1] = 0x39;
	switch (mode) {
	  case AEMODE_FULL_AUTO: buffer[2] = 0x00; break;
	  case AEMODE_MANUAL: buffer[2] = 0x03; break;
	  case AEMODE_SHUTTER: buffer[2] = 0x0A; break;
	  case AEMODE_IRIS: buffer[2] = 0x0B; break;
	  case AEMODE_BRIGHTNESS: buffer[2] = 0x0D; break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "AE Mode")) return;
	if (!WaitComp(camera, "AE Mode", buffer, &len)) return;

	AEMode = mode;
}

void CVisca98::SetECMode(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x3E;
	switch (mode) {
	  case ECMODE_ON: buffer[2] = 0x02; break;
	  case ECMODE_OFF: buffer[2] = 0x03; break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "EC Mode")) return;
	if (!WaitComp(camera, "EC Mode", buffer, &len)) return;

	ECMode = mode;
}

void CVisca98::SetShutter(int camera, int shut)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if ((AEMode != AEMODE_MANUAL) && (AEMode != AEMODE_SHUTTER)) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in current AEMode");
		return;
	}

	if ((shut < 0) || (shut > 0x1B)) {
		CString err;
		err.Format("Shutter out of range (%d)", shut);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Shutter out of range (%d)", shut);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x4A;
	buffer[2] = 0x00;
	buffer[3] = 0x00;
	buffer[4] = (shut & 0x00F0) >> 4;
	buffer[5] = (shut & 0x000F);
	SendMessage(camera, buffer, 6);
	if (!WaitAck(camera, "Shutter")) return;

	if (!WaitComp(camera, "Shutter", buffer, &len)) return;
}


// Mode 0 = reset, 1 = up, 2 = down

void CVisca98::SetBrightness(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (AEMode != AEMODE_BRIGHTNESS) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in this AEMode");
		return;
	}

	if ((mode < 0) || (mode > 2)) {
		CString err;
		err.Format("Brightness out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);	
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Brightness out of range (%d)", mode);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x0D;
	switch (mode) {
	  case 0: buffer[2] = 0x00; break;
	  case 1: buffer[2] = 0x02; break;
	  case 2: buffer[2] = 0x03; break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Brightness")) return;
	if (!WaitComp(camera, "Brightness", buffer, &len)) return;
}

void CVisca98::SetShutterStep(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if ((AEMode != AEMODE_MANUAL) && (AEMode != AEMODE_SHUTTER)){
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in this AEMode");
		return;
	}

	if ((mode < 0) || (mode > 2)) {
		CString err;
		err.Format("Shutter step out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);	
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Shutter step out of range (%d)", mode);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x0A;
	switch (mode) {
	  case 0: buffer[2] = 0x00; break;
	  case 1: buffer[2] = 0x02; break;
	  case 2: buffer[2] = 0x03; break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Shutter Step")) return;
	if (!WaitComp(camera, "Shutter Step", buffer, &len)) return;
}

void CVisca98::SetGainStep(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if(ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}
	if ((AEMode != AEMODE_MANUAL)){
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in this AEMode");
		return;
	}

	if ((mode < 0) || (mode > 2)) {
		CString err;
		err.Format("Gain step out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);	
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Gain step out of range (%d)", mode);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x0C;
	switch (mode) {
	  case 0: buffer[2] = 0x00; break;
	  case 1: buffer[2] = 0x02; break;
	  case 2: buffer[2] = 0x03; break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Gain Step")) return;
	if (!WaitComp(camera, "Gain Step", buffer, &len)) return;
}

void CVisca98::SetGain(int camera, int gain)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if(ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}
	if ((AEMode != AEMODE_MANUAL)){
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in this AEMode");
		return;
	}

	if ((gain < 0) || (gain > 7)) {
		CString err;
		err.Format("Gain out of range (%d)", gain);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Gain out of range (%d)", gain);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x4C;
	buffer[2] = 0;
	buffer[3] = 0; 
	buffer[4] = 0;
	buffer[5] = (gain & 0x000F);
	SendMessage(camera, buffer, 6);
	if (!WaitAck(camera, "Gain")) return;
	if (!WaitComp(camera, "Gain", buffer, &len)) return;

}

void CVisca98::SetIrisStep(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if(ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}
	if ((AEMode != AEMODE_MANUAL) && (AEMode != AEMODE_IRIS)){
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in this AEMode");
		return;
	}

	if ((mode < 0) || (mode > 2)) {
		CString err;
		err.Format("Iris step out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);	
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Iris step out of range (%d)", mode);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x0B;
	switch (mode) {
	  case 0: buffer[2] = 0x00; break;
	  case 1: buffer[2] = 0x02; break;
	  case 2: buffer[2] = 0x03; break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Iris Step")) return;
	if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
}

void CVisca98::SetIris(int camera, int iris)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if(ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}
	if ((AEMode != AEMODE_MANUAL) && (AEMode != AEMODE_IRIS)){
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in this AEMode");
		return;
	}


	if ((iris < 0) || (iris > 0x11)) {
		CString err;
		err.Format("iris out of range (%d)", iris);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Iris out of range (%d)", iris);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x4B;
	buffer[2] = 0;
	buffer[3] = 0; 
	buffer[4] = (iris & 0x00f0) >> 4;
	buffer[5] = (iris & 0x000F);
	SendMessage(camera, buffer, 6);
	if (!WaitAck(camera, "Iris")) return;
	if (!WaitComp(camera, "Iris", buffer, &len)) return;

}


void CVisca98::SetTime(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;
	CTime theTime = CTime::GetCurrentTime();
	int		year, day, month, hour, min;

	
	if (CameraType == EVI_330) {
		if(ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}
	year = theTime.GetYear();
	month = theTime.GetMonth();
	day = theTime.GetDay();
	hour = theTime.GetHour();
	min = theTime.GetMinute();
//	wgPrintf("%d %d %d %d %d", year, month, day, hour, min);

	buffer[0] = 0x07;
	buffer[1] = 0x29;
	buffer[2] = (year % 100) / 10;
	buffer[3] = year % 10;
	buffer[4] = (month % 100) / 10;
	buffer[5] = month % 10;
	buffer[6] = (day % 100) / 10;
	buffer[7] = day % 10;
	buffer[8] = (hour % 100) / 10;
	buffer[9] = hour % 10;
	buffer[10] = (min % 100) / 10;
	buffer[11] = min % 10;
	SendMessage(camera, buffer, 12);
	if (!WaitAck(camera, "Zoom")) return;
	if (!WaitComp(camera, "Zoom", buffer, &len)) return;
	if (TimeDisplay && DateDisplay) {
		SetTimeDisplay(camera, DISPLAY_DATE_TIME);
	}
	if (!TimeDisplay && DateDisplay) {
		SetTimeDisplay(camera, DISPLAY_DATE);
	}
	if (TimeDisplay && !DateDisplay) {
		SetTimeDisplay(camera, DISPLAY_TIME);
	}
	if (!TimeDisplay && !DateDisplay) {
		SetTimeDisplay(camera, DISPLAY_OFF);
	}

}

void CVisca98::SetTimeDisplay(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if(ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable for EVI-330");
		return;
	}
	if ((mode < 0) || (mode > 3)) {
		CString err;
		err.Format("Time display out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);	
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Time display out of range (%d)", mode);
		return;
	}

	switch(mode) {
	case DISPLAY_OFF:
		buffer[0] = 0x07;
		buffer[1] = 0x2A;
		buffer[2] = 0x03;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		buffer[0] = 0x07;
		buffer[1] = 0x2B;
		buffer[2] = 0x03;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		TimeDisplay = false;
		DateDisplay = false;
		break;
	case DISPLAY_TIME:
		buffer[0] = 0x07;
		buffer[1] = 0x2A;
		buffer[2] = 0x03;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		buffer[0] = 0x07;
		buffer[1] = 0x2B;
		buffer[2] = 0x02;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		TimeDisplay = true;
		DateDisplay = false;
		break;
	case DISPLAY_DATE:
		buffer[0] = 0x07;
		buffer[1] = 0x2B;
		buffer[2] = 0x03;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		buffer[0] = 0x07;
		buffer[1] = 0x2A;
		buffer[2] = 0x02;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		TimeDisplay = false;
		DateDisplay = true;
		break;
	case DISPLAY_DATE_TIME:
		buffer[0] = 0x07;
		buffer[1] = 0x2A;
		buffer[2] = 0x02;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		buffer[0] = 0x07;
		buffer[1] = 0x2B;
		buffer[2] = 0x02;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Iris Step")) return;
		if (!WaitComp(camera, "Iris Step", buffer, &len)) return;
		TimeDisplay = true;
		DateDisplay = true;
		break;
	}

}

// Mode 0 = reset, 1 = set, 2 = recall

void CVisca98::Memory(int camera, int position, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if ((position < 0) || (position > 5)) {
		CString err;
		err.Format("Config position out of range (%d)", position);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) AfxMessageBox(err, MB_ICONEXCLAMATION);
		return;
	}

	if ((mode < 0) || (mode > 2)) {
		CString err;
		err.Format("Config mode out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) AfxMessageBox(err, MB_ICONEXCLAMATION);
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x3F;
	buffer[2] = mode;
	buffer[3] = position;
	SendMessage(camera, buffer, 4);
	if (!WaitAck(camera, "Config")) return;
	if (!WaitComp(camera, "Config", buffer, &len)) return;
}

void CVisca98::SetFocus(int camera, int focus, bool wait) 
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (CameraType == EVI_330) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("EVI-330: Unsupported command");
		return;
	}

	if (FocusMode == FOCUS_MODE_AUTO) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Command unavailable in Auto focus mode");
		return;
	}
	
	if (ALLOW_ERROR_MESSAGES) wgPrintf("Desired focus %d", focus);

	if ((focus < 0x1000) || (focus > 0x9FFF)) {
		CString err;
		err.Format("Focus out of range (%d)", focus);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus out of range (%d)", focus);
		return;
	}
//	if ((zoom > 0) && (zoom < 0x3ff)) {
//		zoom++;
//	}

	buffer[0] = 0x04;
	buffer[1] = 0x48;
	buffer[2] = (focus & 0xF000) >> 12;
	buffer[3] = (focus & 0x0F00) >> 8;
	buffer[4] = (focus & 0x00F0) >> 4;
	buffer[5] = (focus & 0x000F);
	SendMessage(camera, buffer, 6);
	if (!WaitAck(camera, "Focus")) return;
	if (wait) {
		if (!WaitComp(camera, "Focus", buffer, &len)) return;
	}
}

void CVisca98::SetFocusIn(int camera, int speed)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;
	
	buffer[0] = 0x04;
	buffer[1] = 0x08;

	if (FocusMode == FOCUS_MODE_AUTO) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus in Auto Mode: Cannot refocus");
		return;
	}

//	if (speed == 0) speed = FocusSpeed;

	if (CameraType == EVI_330) {
		buffer[2] = 0x02;
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focusing in EVI-330");
	}
	if (CameraType == EVI_370) {
		if ((speed < 2) || (speed > 7)) {
			buffer[2] = 0x02;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Focusing in (standard) EVI-370");
		}
		else {
			buffer[2] = 0x20 + speed;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Focusing in (speed = %d) EVI-370", speed);
		}
	}

	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Focus In")) return;

	if (!WaitComp(camera, "Focus In", buffer, &len)) return;

}

void CVisca98::SetFocusOut(int camera, int speed )
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (FocusMode == FOCUS_MODE_AUTO) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus in Auto Mode: Cannot refocus");
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x08;

//	if (speed == 0) speed = FocusSpeed;

	if (CameraType == EVI_330) {
		buffer[2] = 0x03;
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focusing out EVI-330");
	}
	if (CameraType == EVI_370) {
		if ((speed < 2) || (speed > 7)) {
			buffer[2] = 0x03;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Focusing out (standard) EVI-370");
		}
		else {
			buffer[2] = 0x30 + speed;
			if (ALLOW_ERROR_MESSAGES) wgPrintf("Focusing (speed = %d) EVI-370", speed);
		}
	}

	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Focus Out")) return;
	if (!WaitComp(camera, "Focus Out", buffer, &len)) return;
}

void CVisca98::SetFocusStop(int camera)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if (FocusMode == FOCUS_MODE_AUTO) {
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus in Auto Mode: Cannot refocus");
		return;
	}

	buffer[0] = 0x04;
	buffer[1] = 0x08;
	buffer[2] = 0x00;

	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Focus Stop")) return;
	if (!WaitComp(camera, "Focus Stop", buffer, &len)) return;

	return;
}

void CVisca98::SetFocusMode(int camera, int mode)
{
	unsigned char buffer[PACKET_MAX_LENGTH];
	int len;

	if ((mode != FOCUS_MODE_AUTO) && (mode != FOCUS_MODE_MANUAL)) {
		CString err;
		err.Format("Focus mode out of range (%d)", mode);
		dump_packet_ring_buffer_to_file(err);
		if (ALLOW_ERROR_MESSAGES) wgPrintf("Focus mode out of range (%d)", mode);
		return;
	}

	if (GetFocusMode(1) == mode)  {
		return;
	}

	if (CameraType == EVI_330){
		buffer[0]= 0x04;
		buffer[1] = 0x38;
		buffer[2] = 0x10;
		SendMessage(camera, buffer, 3);
		if (!WaitAck(camera, "Focus")) return;
		if (!WaitComp(camera, "Focus", buffer, &len)) return;
		
		FocusMode = mode;
		return;
	}

	buffer[0]= 0x04;
	switch (mode) {
	  case FOCUS_MODE_AUTO:
		buffer[1] = 0x38;
		buffer[2] = 0x02;
		break;
	  case FOCUS_MODE_MANUAL:
		buffer[1] = 0x38;
		buffer[2] = 0x03;
		break;
	}
	SendMessage(camera, buffer, 3);
	if (!WaitAck(camera, "Focus")) return;
	if (!WaitComp(camera, "Focus", buffer, &len)) return;
	FocusMode = mode;

	return;
}


void CVisca98::init_packet_ring_buffer()
{
	for (RBPTR = RING_BUFFER_LEN - 1; RBPTR > 0; RBPTR--)
		PACKET_RING_BUFFER[RBPTR] = (char)0;
	RBPTR = 0;
	return;
}

void CVisca98::add_to_packet_ring_buffer(char *packetstring)
{
	int i, len;
	len = strlen(packetstring);
	for (i=0; i < len; i++) {
		PACKET_RING_BUFFER[RBPTR] = packetstring[i];
		RBPTR = RBSAFE(RBPTR+1);
	}
	PACKET_RING_BUFFER[RBPTR++] = 0; 
	return;	
}


void CVisca98::dump_packet_ring_buffer_to_file(CString errmsg)
{
	FILE *fp;
	int i, rb, npack, rb1, rb2;
	char bufstr[PACKET_MAX_LENGTH*3+7];

	fp = fopen(ERROR_FILENAME,"a");

    CTime theTime = CTime::GetCurrentTime();
	CString s = theTime.Format( "%A, %B %d, %Y at %X" );

	fprintf(fp,"visca error occurred on %s\n",s);
	fprintf(fp,"error message: %s\n",errmsg);
	fprintf(fp,"dumping last %d packets\n",NUM_PACKETS_TO_DUMP);
	rb2 = RBSAFE(RBPTR);
	for (npack = 0; npack < NUM_PACKETS_TO_DUMP; npack++) {
		rb1 = RBSAFE(rb2 - 2);
		while (PACKET_RING_BUFFER[rb1] != 0) {
			rb1 = RBSAFE(rb1 - 1);
		}
		for (i = 0, rb = RBSAFE(rb1+1); rb != rb2; ) {
			bufstr[i] = PACKET_RING_BUFFER[rb];
			i++;
			rb = RBSAFE(rb+1);
		}
		fprintf(fp,"packet n-%02d: %s\n",npack,bufstr);
		rb2 = RBSAFE(rb1+1);
	}
	fclose(fp);
	return;
}



