// CVisca98 - new visca control code
// By Alan Lipton (based on code by Mike Blackwell and Bob Collins)
//
// Copyright July 1998 - Carnegie Mellon University. All rights reserved
//

#ifndef	__CVISCA98_H__
#define	__CVISCA98_H__




#include "CSerial.h"
#include "resource.h"

#define VISCA_PORT_MODE "baud=9600 parity=N data=8 stop=1"

const int PACKET_MAX_LENGTH = 20;
const int PACKET_TIMEOUT = 1000;	// Wait up to 5 seconds



// GLOBAL VARIABLES - Added by Bob Collins July 6, 1998
// 1) Toggle writing packets to debug window
// 2) toggle turning error messages on or off
// 3) writing error packets to a file for later review


// 10 packets * (3 ascii chars per packet byte + 7 chars for "send :" + '0' end of string byte)
#define RBLEN  (10 * (3 * PACKET_MAX_LENGTH + 7))
#define RBSAFE(a) (((a)<0) ? (RING_BUFFER_LEN-1) : (((a)>=RING_BUFFER_LEN) ? 0 : (a)))

#define	UNKNOWN_CAMERA	(0)
#define	EVI_330		(1)
#define	EVI_370		(2)

#define	FOCUS_MODE_MANUAL	(1)
#define	FOCUS_MODE_AUTO		(0)

#define	WBMODE_AUTO			(0)
#define	WBMODE_INDOOR		(1)
#define	WBMODE_OUTDOOR		(2)
#define	WBMODE_ONE_PUSH		(3)

#define	AEMODE_FULL_AUTO	(0)
#define	AEMODE_MANUAL		(1)
#define	AEMODE_SHUTTER		(2)
#define	AEMODE_IRIS			(3)
#define	AEMODE_BRIGHTNESS	(4)

#define	ECMODE_ON			(1)
#define	ECMODE_OFF			(0)

#define	VALUE_RESET			(0)
#define VALUE_UP			(1)
#define	VALUE_DOWN			(2)

#define	DISPLAY_OFF			(0)
#define	DISPLAY_TIME		(1)
#define	DISPLAY_DATE		(2)
#define	DISPLAY_DATE_TIME	(3)



class CVisca98 {
private:
	CSerial *port;

	void FreePort(void);
	BOOL InitializePort(LPCSTR portName);
	char PACKET_RING_BUFFER[RBLEN];
	char ERROR_FILENAME[256];
	int RBPTR, RING_BUFFER_LEN , NUM_PACKETS_TO_DUMP;
	void init_packet_ring_buffer();
	void add_to_packet_ring_buffer(char *packetstring);
	void dump_packet_ring_buffer_to_file(CString errmsg);
	bool Initialize(LPCSTR portName);


public:
	CString PortName;
	int DEBUG_PACKETS;
	int ALLOW_ERROR_MESSAGES;
	bool ViscaOpen;
	int		CameraType;
	int		Zoom;
	int		ZoomSpeed;
	int		FocusMode;
	int		Focus;
	int		FocusSpeed;
	int		WBMode;
	int		AEMode;
	int		ECMode;
	int		Shutter;
	int		Iris;
	int		Gain;
	bool	KeyLock;
	bool	TimeDisplay;
	bool	DateDisplay;

	bool GetCameraState();

	CVisca98(void);
	~CVisca98(void);

	bool Open(LPCSTR portName);
	bool Close();
	BOOL SendMessage(int dest, const unsigned char *data, int length,
		             int format = 1);
	void SendInquiry(int dest, unsigned char inq);
	BOOL RecvMessage(int *src, unsigned char *data, int *length);
	void ShowMessage(int src, unsigned char *data, int length);
// LIPTON'S NEW VERSION
	int IdentifyCamera(int camera);

	BOOL WaitAck(int dest, LPCSTR command);
	BOOL WaitComp(int dest, LPCSTR command, unsigned char *data, int *length);

	bool GetKeyLock(int camera);
	int GetFocusMode(int camera);
	int GetWBMode(int camera);
	int GetZoom(int camera);
	int GetFocus(int camera);
	int GetAEMode(int camera);
	int GetECMode(int camera);
	int GetShutter(int camera);
	int GetIris(int camera);
	int GetGain(int camera);

	void SetKeyLock(int camera, BOOL on);

	void SetWBTrigger(int camera);
	void SetWBMode(int camera, int mode);

	void SetZoom(int camera, int zoom, bool wait = false);
	void SetZoomIn(int camera, int speed = 0);
	void SetZoomOut(int camera, int speed = 0);
	void SetZoomStop(int camera);

	void SetFocus(int camera, int focus, bool wait = false);
	void SetFocusIn(int camera, int speed = 0);
	void SetFocusOut(int camera, int speed = 0);
	void SetFocusStop(int camera);
	void SetFocusMode(int camera, int mode);

	void SetAEMode(int camera, int mode);
	void SetECMode(int camera, int mode);
	void SetShutter(int camera, int shut);
	void SetShutterStep(int camera, int mode);
	void SetBrightness(int camera, int mode);
	void SetGainStep(int camera, int mode);
	void SetGain(int camera, int gain);
	void SetIrisStep(int camera, int mode);
	void SetIris(int camera, int iris);

	void SetTime(int camera);
	void SetTimeDisplay(int camera, int mode);

	void Memory(int camera, int position, int mode);
};




#endif	// __CVISCA98_H__